@@ -34,6 +34,7 @@
 #include <ostream>
 #include <streambuf>
 
+#include <atf-c++/fs.hpp>
 #include <atf-c++/utils.hpp>
 
 namespace atf {
@@ -232,7 +233,7 @@ private:
     //! constant because, in the latter case, we cannot define it under
     //! Win32 due to the value being of a complex type.
     //!
-    static const handle_type invalid_value(void);
+    static handle_type invalid_value(void);
 };
 
 // ------------------------------------------------------------------------
@@ -631,6 +632,8 @@ public:
 //!
 unbuffered_istream& getline(unbuffered_istream&, std::string&);
 
+int cmp(const fs::path&, const fs::path&);
+
 } // namespace io
 } // namespace atf
 
