@@ -1,7 +1,7 @@
 /*
  * Automated Testing Framework (atf)
  *
- * Copyright (c) 2008 The NetBSD Foundation, Inc.
+ * Copyright (c) 2008, 2009 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,12 +34,14 @@
 #include <stdbool.h>
 
 #include <atf-c/error_fwd.h>
+#include <atf-c/list.h>
 
 atf_error_t atf_text_for_each_word(const char *, const char *,
                                    atf_error_t (*)(const char *, void *),
                                    void *);
 atf_error_t atf_text_format(char **, const char *, ...);
 atf_error_t atf_text_format_ap(char **, const char *, va_list);
+atf_error_t atf_text_split(const char *, const char *, atf_list_t *);
 atf_error_t atf_text_to_bool(const char *, bool *);
 atf_error_t atf_text_to_long(const char *, long *);
 
